% Software for the evaluation of laser-induced stripes in cell nuclei.
% Negative Stripes
% 
% The method is described here:
% Martin Mistrik, Eva Vesela, et al:
% Cells and Stripes: A novel quantitative photo-manipulation technique
% Scientific Reports, to appear.
% 
% Copy the TIF files you want to analyze into the 'data' folder.
% Single layer gray-scale TIFF files in 16bit depth are expected.
% The files must be named so that they are in the correct order, 
% i.e. the first image is taken before the stripes are induced.

%% USER EDIT
pars = zadavani_dna; 

% pars.area     (minimum area for a nucleus to be recognized)
% pars.solid    (minimum solidity for a nucleus to be recognized)
% pars.sopel    (the distance between the stripes)
% pars.polomer  (half-width of each stripe)

pars.tolerance = 0.3; % percent tolerance for the change in the position of the nucleus for the tracking 
pars.sekvence = [0.1 0.2 0.4 0.6 0.8 1 1.2 1.4 1.6 1.8]; % threshold sequence

%% BEGIN READ DATA

S = dir('Data\*.tif*');
names = {S.name};
if length(names)<2
    error('At least two files are expected!')
end

disp('**************************************************')
disp('Before irradiation:')
disp(names(1))
disp('After irradiation:')
disp((names(2:end))')
disp('**************************************************')

%% READ AND SEGMENT THE FIRST FILE

celkvysl = struct([]); % allocate for the results structure

tic
disp('Segmenting the first file...')
vysl = nactiprvni_dna1(names,pars); % segment the first file
% this is identical to the positive mode
toc
% vysl.name (filename)
% vysl.data ([area, xcenter, ycenter, solidity, rows, columns] of the recognized objects)
%           (rows and columns are addresses of all pixels in the object)
% vysl.mask (logical mask with the recognized objects)
n = length(vysl.data);
disp(['First file: ' num2str(n) ' objects identified.'])
disp('done')

celkvysl(1).name = vysl.name;
celkvysl(1).data = vysl.data;
celkvysl(1).mask = vysl.mask;

%% READ AND SEGMENT THE SUBSEQUENT FILES

for i=2:1:length(names)
    
    name = names(i); % current file
    disp('****************************************************')
    disp(['Segmenting file ' char(name) '...'])

    tic
    minvysl = doplnvysl_dna(celkvysl,i);
    % if nothing found at previous times, look into more distant past
    vysl = tracking_frap1(minvysl,pars,name);
    toc
    % vysl.name (filename)
    % vysl.data ([area, xcenter, ycenter, solidity, rows, columns] of the recognized objects)
    %           (rows and columns are addresses of all pixels in the object)
    % vysl.mask (logical mask with the recognized objects)


    pom = vysl.data;
    pom = [pom.area];
    n = sum(pom>0);

    disp(['File ' char(name) ': ' num2str(n) ' objects identified.'])
    disp('done.')
    
    celkvysl(i).name = vysl.name;
    celkvysl(i).data = vysl.data;
    celkvysl(i).mask = vysl.mask;

end

save('segmentdata.mat','celkvysl','pars');
disp('Writing results of segmentation to file segmentdata.mat.')
disp('****************************************************')

