% Software for the evaluation of laser-induced stripes in cell nuclei.
% Negative Stripes
% 
% The method is described here:
% Martin Mistrik, Eva Vesela, et al:
% Cells and Stripes: A novel quantitative photo-manipulation technique
% Scientific Reports, to appear.

% Called from 'prouzky_frap.m'
% Main file: BNeg_Evaluate_Stripes, see the documentation there

% Selects the optimal rotation of an object

function argout = hloubka_frap(argin1,argin2)

Rtreat = argin1; % radon trafo of an object
kde = argin2; % local minima in Rtreat (linear index)

% the most probable rotation is a deepest valley
% with the longest banks

[ii jj] = ind2sub(size(Rtreat),kde); % subs
vals = zeros(length(ii),1); % allocate for the valley depth
[mm nn] = size(Rtreat);

for k=1:1:length(ii)

    rad = ii(k); sloup = jj(k); s1=0; h1 = Rtreat(rad,sloup);
    while rad<mm && Rtreat(rad+1,sloup)>Rtreat(rad,sloup)
        rad = rad+1;
        s1 = s1+1;
    end
    h1 = Rtreat(rad,sloup) - h1; % depth
    
    rad = ii(k); sloup = jj(k); s2=0; h2 = Rtreat(rad,sloup);
    while rad>1 && Rtreat(rad-1,sloup)>Rtreat(rad,sloup)
        rad = rad-1;
        s2 = s2+1;
    end
    h2 = Rtreat(rad,sloup) - h2; % depth
    
    rad = ii(k); sloup = jj(k); s3=0; h3 = Rtreat(rad,sloup);
    while sloup<nn && Rtreat(rad,sloup+1)>Rtreat(rad,sloup)
        sloup = sloup+1;
        s3 = s3+1;
    end
    h3 = Rtreat(rad,sloup) - h3; % depth
    
    rad = ii(k); sloup = jj(k); s4=0; h4 = Rtreat(rad,sloup);
    while sloup>1 && Rtreat(rad,sloup-1)>Rtreat(rad,sloup)
        sloup = sloup-1;
        s4 = s4+1;
    end
    h4 = Rtreat(rad,sloup) - h4; % depth
    
    vals(k) = mean([h1 h2 h3 h4]); % average
    
    
end

argout = vals;
