% Software for the evaluation of laser-induced stripes in cell nuclei.
% Negative Stripes
% 
% The method is described here:
% Martin Mistrik, Eva Vesela, et al:
% Cells and Stripes: A novel quantitative photo-manipulation technique
% Scientific Reports, to appear.

% Main file: Segment_and_Track, see the documentation there

% This functions segments the first file.
% It uses local thresholding in a moving window
% See the important parameter on line 45!
% Moreover, there is a median filter in 'segmentuj_dna1.m' that you may want to play with

function argout = nactiprvni_dna1(argin1,argin2)

names = argin1; % list of filenames in the Data folder
pars = argin2;
% pars.area     (minimum area for a nucleus to be recognized)
% pars.solid    (minimum solidity for a nucleus to be recognized)
% pars.sopel    (the distance between the stripes)
% pars.polomer  (half-width of each stripe)

mkdir('Preview')
jm = ['Data\' char(names(1))];
prv = imread(jm,'tif');
prv = imfill(prv,'holes'); % an important trick
prvold = prv;

%% convert from 16bit to 8bit
% this makes Otsu perform much better

dol = quantile(prv(:),.01); % 1% quantile
hor = quantile(prv(:),.99); % 99% quantile
prv1 = double(prv);
prv1 = prv1 - double(dol);
prv1 = prv1./double(hor-dol);
prv1(prv1>1) = 1;
prv1 = uint8(255*prv1);
prv = prv1;

%% moving window thresholding

objekty = struct(); % allocation
posobjekty = 0;
pars.del = 150; % window size
% Should be large enough so that even the biggest nucleus fits into a single window.
% Should be small enough no to hide low intensity objects behind high intensity ones.
pars.krok = floor(pars.del/3); % move the window with this step
[mm nn] = size(prv);
radky = [1:pars.krok:(mm-pars.del) mm-pars.del];
sloupce = [1:pars.krok:(nn-pars.del) nn-pars.del];
for rad = radky
    for sloup = sloupce

        kus = prv(rad:(rad+pars.del),sloup:(sloup+pars.del)); % single window
        mask = segmentuj_dna1(kus,pars); % segment a single window
        % this function uses a median filter that you may want to play with

        [L num] = bwlabel(mask);
        stats = regionprops(L, 'Area', 'Centroid', 'Solidity', 'PixelIdxList');
        for k=1:1:num
            posobjekty = posobjekty + 1;
            objekty(posobjekty).area = stats(k).Area;
            objekty(posobjekty).xcenter = stats(k).Centroid(1) + sloup -1;
            objekty(posobjekty).ycenter = stats(k).Centroid(2) + rad -1;
            objekty(posobjekty).solid = stats(k).Solidity;
            pil = stats(k).PixelIdxList;
            [rr ss] = ind2sub(size(mask),pil); 
            objekty(posobjekty).rad = rad + rr - 1; % rows in the original
            objekty(posobjekty).sloup = sloup + ss - 1; % columns in the original
        end
    end
end

% now 'objekty' need cleaning up because each object appears many times
ktere = vycisti_dna(objekty);
% retruns the indices of each object only once
% quick and dirty, could be much improved
objekty1 = objekty(ktere); % only the selected ones, hopefully each one only once
% objekty1: [area, xcenter, ycenter, solidity, rows, columns]

maska = false(size(prv)); %allocate for the mask
for i=1:1:length(objekty1)
    ii = objekty1(i).rad;
    jj = objekty1(i).sloup;
    ind = sub2ind(size(maska),ii,jj);
    maska(ind) = true;
end

bound = bwperim(maska);
h = figure;
imshow(prvold,[0 max(prvold(:))])
hold on
[ii jj] = find(bound);
plot(jj,ii,'r.','MarkerSize',1)
title(['Segmented image ' char(names(1)) ', no. of objects = ' num2str(length(objekty1))])
imjm = ['Preview\' char(names(1)) '.png'];
saveas(h,imjm,'png');

vysl.name = char(names(1)); % name
vysl.data = objekty1; % chars of recognized objects
vysl.mask = maska; % the whole mask

argout = vysl;

