% Software for the evaluation of laser-induced stripes in cell nuclei.
% Negative Stripes
% 
% The method is described here:
% Martin Mistrik, Eva Vesela, et al:
% Cells and Stripes: A novel quantitative photo-manipulation technique
% Scientific Reports, to appear.

% Called from 'prouzky_frap.m'
% Main file: BNeg_Evaluate_Stripes, see the documentation there

% Selects the optimal starting point for stripe fitting


function argout = najdistart_frap(argin)

proj = argin; % downward projection

%% napred najit lokalni minima

pks = imregionalmin(proj); % improcessing
pks(proj==0) = false;
locs = find(pks);
pks = proj(locs);


vals = zeros(length(pks),1); 
m = length(proj);

for k=1:1:length(pks) % over all candidates

    pos = locs(k); 
    s1=0; 
    h1 = proj(pos);
    while pos<m && proj(pos+1)>proj(pos) 
        pos = pos+1;
        s1 = s1+1;
    end
    h1 = proj(pos) - h1; 
    
    pos = locs(k); 
    s2=0; 
    h2 = proj(pos);
    while pos>1 && proj(pos-1)>proj(pos) 
        pos = pos-1;
        s2 = s2+1;
    end
    h2 = proj(pos) - h2; 
    
    vals(k) = min([h1 h2]);
    
end

kde = find(vals==max(vals));
argout = locs(kde); % position of the deepest valley

