% Software for the evaluation of laser-induced stripes in cell nuclei.
% Negative Stripes
% 
% The method is described here:
% Martin Mistrik, Eva Vesela, et al:
% Cells and Stripes: A novel quantitative photo-manipulation technique
% Scientific Reports, to appear.

% Called from: tracking_frap1.m
% Main file: 1Neg_Segment_and_Track.m
% A simple routine which tests a cascade of thresholds and selects the
% optimal one.

function [argout1 argout2] = posledni_frap(argin1, argin2, argin3)

% argout1 = mask of the recognized object
% argout2 = status, i.e. the real threshold or zero

kus = argin1; % a clip to be segmented
plocha = argin2; % area of the object in the clip in previous time
pars = argin3;
    % pars.area     (minimum area for a nucleus to be recognized)
    % pars.solid    (minimum solidity for a nucleus to be recognized)
    % pars.sopel    (the distance between the stripes)
    % pars.polomer  (half-width of each stripe)
    % pars.sekvence (the threshold cascade)

%% BODY

% use a cascade of thresholds based on Otsu
% select the one which makes the results most solid
% and most similar to the object at previous time.

% This is less intelligent than the positive version, however, works well
% here

chars = zeros(5,length(pars.sekvence)); % allocation
% chars(1,:) anything in the middle?
% chars(2,:) solidity of the object in the middle
% chars(3,:) relativ change in area of the object in the middle
% chars(4,:) thresh value
% chars(5,:) anything touching the boundary?
poschars = 0;
[mm nn] = size(kus);

level = graythresh(kus); % otsu automatic suggestion
level = round(2^16*level); % in 16bitu
sekvence = level*pars.sekvence;
delej=true; 

for thresh = sekvence % cascade of thresh
    if delej
    poschars = poschars + 1;
    mask = kus>=thresh;
    mask = imfill(mask,'holes');
    L = bwlabel(mask);
    stats = regionprops(L,'Solidity','Area','PixelIdxList');
    co = L(floor(size(L,1)/2),floor(size(L,2)/2)); % what is in the middle?
    chars(1,poschars) = (co>0); 
    if co>0 % anything there
        newsolidita = stats(co).Solidity;
        chars(2,poschars) = newsolidita;
        newplocha = stats(co).Area;
        zmena = abs(newplocha - plocha)/plocha;
        chars(3,poschars) = zmena;
        pom = stats(co).PixelIdxList;
        [ii jj] = ind2sub(size(kus),pom);
        chars(5,poschars) = (min(ii)==1) || (min(jj)==1) || (max(ii) == mm) || (max(jj) == nn); % dotek
        
    else
        delej=false; % no use continuing
        chars(3,poschars) = 10; % maximum

    end
    chars(4,poschars) = thresh; % threshold value
    end %if

end

% among those with acceptable relative area change (20%)
% select the most solid one
kde1 = chars(1,:)==0; % nothing there
kde2 = chars(3,:)>pars.tolerance; % too large area change
kde3 = chars(5,:)==1; % touching the boundary
kde4 = chars(2,:)<pars.solid; % too ugly
kde = kde1 | kde2 | kde3 | kde4;
chars(:,kde)=[]; % delete them all

if size(chars,2)>0 % anything left?
    kde = find(chars(2,:)==max(chars(2,:))); % where is max solidity
    kde = kde(1);
    prah = chars(4,kde); % this is optimal

    mask = kus>=prah;
    mask = imfill(mask,'holes');
    mask = bwmorph(mask,'open'); 
    L = bwlabel(mask);
    co = L(floor(size(L,1)/2),floor(size(L,2)/2)); % what is in the middle
    if co==0 % some error
        argout1 = false(size(kus)); % black mask
        argout2 = 0;
    else

        mask = (L==co);

        argout1 = mask;
        argout2 = prah;

    end

else
    argout1 = false(size(kus)); % black mask
    argout2 = 0;
end

    




