% Software for the evaluation of laser-induced stripes in cell nuclei.
% Negative Stripes
% 
% The method is described here:
% Martin Mistrik, Eva Vesela, et al:
% Cells and Stripes: A novel quantitative photo-manipulation technique
% Scientific Reports, to appear.

% Main file: BNeg_Evaluate_Stripes, see the documentation there

% Computes the characteristics of an object

function [argout1 argout2] = prouzky_frap(argin1,argin2,argin3,argin4)
    
treat = argin1; % a clip with the object without background
treatmask = argin2; % mask of the object (logical)
pars = argin3; 
    % pars.area     (minimum area for a nucleus to be recognized)
    % pars.solid    (minimum solidity for a nucleus to be recognized)
    % pars.sopel    (the distance between the stripes)
    % pars.polomer  (half-width of each stripe)
    % pars.sekvence (threshold sequence)
orot = argin4; % suggested rotation from the previous time

% argout1 = [SID_object Area_object SID_stripes Area_stripes MS]
% argout2 = current angle of rotation


%% ROTATION

treatfilt = medfilt2(treat,[3,3]);
fli = (orot-30):1:(orot+30); % reasonably small range!
Rtreat = radon(treatfilt,fli); % radon trafo of the image
Rbw = imregionalmin(Rtreat); % local minima
Rbw = Rbw & (Rtreat>0); % only inside
Rbw(:,1) = false; Rbw(:,end)=false; % not on the boundary
kde = find(Rbw); % candidates for the correct rotation

% a more complicated version of looking for the correct rotation
vals = hloubka_frap(Rtreat,kde); 
ktere = find(vals==max(vals));
if numel(ktere)==0 
    uhel = -orot;
else
    pos = kde(ktere);
    [ii jj] = ind2sub(size(Rtreat),pos);
    uhel = -fli(jj(1)); 
end
argout2 = -uhel;
treatrot = imrotate(treat,uhel); % stripes vertical
maskrot =  imrotate(treatmask,uhel);

proj = sum(treatrot);
pos = najdistart_frap(proj); % optimal starting point for stripe fitting
if isempty(pos)
    argout1 = NaN(1,4);
    return
end


%% FIND STRIPES

D = (pars.sopel-5):0.1:(pars.sopel+5);
S = pars.polomer;
kde = pos-2:1:pos+2;
hods = zeros(length(D),length(kde));

for rad = 1:1:length(D);
    for sloup = 1:1:length(kde);

        vstup = [D(rad) S kde(sloup)];
        pruch =  opruhuj_frap(vstup,treatrot,maskrot);
        % pruch = [SID_object Area_object SID_stripes Area_stripes MS]
        krit = (pruch(3)/pruch(4))/(pruch(1)/pruch(2)); % relative intensity in stripes
        hods(rad,sloup) = krit; % ratio of intensity inside and outside of strip

    end
end

[rad sloup] = find(hods==min(hods(:))); % minimum relative intensity
rad = rad(1); sloup = sloup(1);
vstup = [D(rad) S kde(sloup)];
pruch =  opruhuj_frap(vstup,treatrot,maskrot);
% pruch = [SID_object Area_object SID_stripes Area_stripes MS]
argout1 = pruch; 

