% Software for the evaluation of laser-induced stripes in cell nuclei.
% Negative Stripes
% 
% The method is described here:
% Martin Mistrik, Eva Vesela, et al:
% Cells and Stripes: A novel quantitative photo-manipulation technique
% Scientific Reports, to appear.

% Main file: 1Neg_Segment_and_Track.m
% A simple routine which tests a cascade of thresholds and selects the
% optimal one.

function argout = tracking_frap1(argin1,argin2,argin3)

vysl = argin1; % results of segmentation at the previous time
    % vysl.name (filename)
    % vysl.data ([area, xcenter, ycenter, solidity, rows, columns] of the recognized objects)
    %           (rows and columns are addresses of all pixels in the object)
    % vysl.mask (logical mask with the recognized objects)

pars = argin2;
    % pars.area     (minimum area for a nucleus to be recognized)
    % pars.solid    (minimum solidity for a nucleus to be recognized)
    % pars.sopel    (the distance between the stripes)
    % pars.polomer  (half-width of each stripe)
    % pars.sekvence (the threshold cascade)

name = argin3; % which file to segment now (cell)
% argout has the same structure as 'vysl'


%% 

jm = ['Data\' char(name)];
dru = imread(jm,'tif');
dru = medfilt2(dru,[5,5]); % median filter
dru = imfill(dru,'holes'); % important trick
[mm nn] = size(dru);

objekty = struct(); % allocation for the result

for j=1:1:length(vysl.data) % over all recognized objects at time 0
    
    plocha = vysl.data(j).area; % previous area of the object

    if plocha > 0 % if object is not lost
    
        rad =  vysl.data(j).rad;
        sloup = vysl.data(j).sloup;
        rroz = max(rad) - min(rad);
        sroz = max(sloup) - min(sloup);
        odr = max(floor(min(rad) - 0.5*rroz),1);
        dor = min(floor(max(rad) + 0.5*rroz),mm);
        ods = max(floor(min(sloup) - 0.5*sroz),1);
        dos = min(floor(max(sloup) + 0.5*sroz),nn);
        kus = dru(odr:dor,ods:dos); % clip in which to search for object j

        [mask status] = posledni_frap(kus,plocha,pars);
        if status > 0 

            [L num] = bwlabel(mask); % num must be 1!
            stats = regionprops(L, 'Area', 'Centroid', 'Solidity', 'PixelIdxList');
            objekty(j).area = stats(1).Area;
            objekty(j).xcenter = stats(1).Centroid(1) + ods -1;
            objekty(j).ycenter = stats(1).Centroid(2) + odr -1;
            objekty(j).solid = stats(1).Solidity;
            pil = stats(1).PixelIdxList;
            [rr ss] = ind2sub(size(mask),pil); 
            objekty(j).rad = odr + rr - 1; % rows in the original image 
            objekty(j).sloup = ods + ss - 1; % columns in the original image
            
        else % tracking failed
            
            objekty(j).area=0; % nothing
            
        end
        
    else % lost long ago
        
        objekty(j).area = 0; 

    end % if 
    
end % for 

pocet = 0;
maska = false(size(dru)); %allocation for the mask
for i=1:1:length(objekty)
    if objekty(i).area>0
        ii = objekty(i).rad;
        jj = objekty(i).sloup;
        ind = sub2ind(size(maska),ii,jj);
        maska(ind) = true;
        pocet = pocet+1;

    end
end

bound = bwperim(maska);
h = figure;
imshow(dru,[0 max(dru(:))])
hold on
[ii jj] = find(bound);
plot(jj,ii,'r.','MarkerSize',1)
title(['Segmented image ' char(name) ' no. of object = ' num2str(pocet)])
imjm = ['Preview\' char(name) '.png'];
saveas(h,imjm,'png');

vyslnew.name = char(name);
vyslnew.data = objekty;
vyslnew.mask = maska;

argout = vyslnew;
    

    
     
    
    
    
