% Software for the evaluation of laser-induced stripes in cell nuclei.
% Positive Stripes
% 
% The method is described here:
% Martin Mistrik, Eva Vesela, et al:
% Cells and Stripes: A novel quantitative photo-manipulation technique
% Scientific Reports, to appear.

% Called from: prouzky_dna4
% Main file: Evaluate_Stipes.m, see the documentation there

% Computes the characteristics of the rotated object
% for suggested gauge and position of the stripes

function argout = opruhuj_dna3(argin1,argin2,argin3)

D = argin1(1); % gauge
S = argin1(2); % stripe half-width
x0 = argin1(3); % where to start

% argout = [SID_object Area_object SID_stripes Area_stripes MS]

treatrot = argin2; % rotated clip
maskrot = argin3; % rotated mask

% CYA
[mm nn] = size(treatrot);
if x0<1
    x0=1;
elseif x0>nn
    x0=nn;
end


pruhy = false(1,nn); % allocation for the indicator of stripes
pruhy(x0) = true;
pos = x0:D:nn;
pos = round(pos);
pruhy(pos) = true;
pos = x0:(-D):1;
pos = round(pos);
pruhy(pos) = true;
kde = find(pruhy);
for i=1:1:length(kde)
    pos = kde(i);
    od = max(pos - S,1);
    do = min(pos + S,nn);
    pruhy(od:do)=true;
end

pom = treatrot(:,pruhy);
sig1 = sum(pom(:)); % SID in stripes
sig2 = sum(treatrot(:)); % total SID

pom = maskrot(:,pruhy);
mas1 = sum(pom(:)); % area in stripes
mas2 = sum(maskrot(:)); % total area

teor = (double(mas1)/double(mas2))*double(sig2); % expected signal in stripes
prakt = double(sig1); % real signal in stripes
MS = (prakt - teor)/teor;  % measure of striation
%MS = prakt - teor; % absolute version of MS, not used

argout = [double(sig2) double(mas2) double(sig1) double(mas1) MS];
% [SID_object Area_object SID_stripes Area_stripes MS]

    
    