% Software for the evaluation of laser-induced stripes in cell nuclei.
% Positive Stripes
% 
% The method is described here:
% Martin Mistrik, Eva Vesela, et al:
% Cells and Stripes: A novel quantitative photo-manipulation technique
% Scientific Reports, to appear.

% Called from: tracking_dna1.m
% Main file: Segment_and_Track, see the documentation there

% A nonstandard interval bisection threshold method
% Attention, line 41 will be sensitive to different image types!

function [argout1 argout2] = posledni_dna1(argin1, argin2, argin3)

% argout1 = mask of the recognized object
% argout2 = status, i.e. the real threshold or zero

kus = argin1; % a clip to be segmented
plocha = argin2; % area of the object in the clip in previous time
pars = argin3;
    % pars.area     (minimum area for a nucleus to be recognized)
    % pars.solid    (minimum solidity for a nucleus to be recognized)
    % pars.sopel    (the distance between the stripes)
    % pars.polomer  (half-width of each stripe)


%% BODY

level = graythresh(kus); % otsu automatic threshold as a first guess
horni = round(2*(2^16*level)); % upper bound for true thresh
dolni = quantile(kus(:),.05); % lower bound for true thresh
delej = true; % continue looking for the optimal thresh
[m n] = size(kus);
pocet = 0; % how many iterations

while delej % until object found or until failure
    pocet = pocet+1;
    
    if (pocet > 30) || ((horni - dolni) < 100) % failure
        delej = false;
        nasel = false; % nothing found
    end

    level = floor(0.5*(horni+dolni)); % current thresh
    aktmask = kus>level;
    [L num] = bwlabel(aktmask,4); 
    co = aktmask(floor(size(L,1)/2),floor(size(L,2)/2)); % anything in the middle?

    if (co==0) % thresh too large
        horni = level; % go down
    else % somethig's there
        stats = regionprops(L,'Area','Solidity','PixelIdxList');
        co = L(floor(size(L,1)/2),floor(size(L,2)/2)); % what is in the middle
        newsolidita = stats(co).Solidity; % solidity of object in the middle
        newplocha = stats(co).Area;
        zmena = (newplocha - plocha)/plocha; % relative area change
        pom = stats(co).PixelIdxList;
        [ii jj] = ind2sub(size(kus),pom);
        dotek = (min(ii)==1) || (min(jj)==1) || (max(ii) == m) || (max(jj) == n); % touching boundary
        if dotek || (zmena > pars.tolerance)
            % touching boundary or too large => lower thresh
            dolni = level; 
        elseif (zmena < -pars.tolerance) || (newsolidita < pars.solid)
            % too small or too ugly
            % cannot be touching boundary, cannot be too large
            % raise thresh
            horni = level;
        else % no touching boundary, area change OK, solidity OK
            nasel = true;
            delej = false;
        end % if dotek
    end % if (co==0)
end % while
            
if nasel % found
    vysl = (L==co);
    vysl = imfill(vysl,'holes');
    argout1 = vysl;
    argout2 = true;
else
    argout1 = false(size(kus)); % black mask
    argout2 = false;
end

    




