% Software for the evaluation of laser-induced stripes in cell nuclei.
% Positive Stripes
% 
% The method is described here:
% Martin Mistrik, Eva Vesela, et al:
% Cells and Stripes: A novel quantitative photo-manipulation technique
% Scientific Reports, to appear.

% Main file: Evaluate_Stipes.m, see the documentation there

% Computes the characteristics of an object

function [argout1 argout2] = prouzky_dna4(argin1,argin2,argin3,argin4)
    
treat = argin1; % a clip with the object without background
treatmask = argin2; % mask of the object (logical)
pars = argin3; 
    % pars.area     (minimum area for a nucleus to be recognized)
    % pars.solid    (minimum solidity for a nucleus to be recognized)
    % pars.sopel    (the distance between the stripes)
    % pars.polomer  (half-width of each stripe)
orot = argin4; % suggested rotation from the previous time

% argout1 = [SID_object Area_object SID_stripes Area_stripes MS]
% argout2 = current angle of rotation

%% ROTATION
% rotate the image so that the (possible) stripes are vertical

cotocit = treat; % clip
fli = (orot-90):1:(orot+90); % possible rotations
radontraf = radon(cotocit,fli); % Radon Transform
[ii jj] = find(radontraf == max(radontraf(:))); % simple max
uhel = fli(jj(1)); % optimal angle of rotation
argout2 = uhel;

treatrot = imrotate(treat,-uhel); % stripes vertical
maskrot =  imrotate(treatmask,-uhel); 

proj = sum(treatrot); % sum of columns
pos = find(proj==max(proj)); % highest peak


%% FIND STRIPES

D = (pars.sopel-10):1:(pars.sopel+10); % possible gauges (the cells are living and move!)
S = pars.polomer; % fixed half-width of the stripe
kde = pos-2:1:pos+2; % where to start looking for the first stripe
hods = zeros(length(D),length(kde)); % aloocation

% the most simple optimisation: see all possibilites
for rad = 1:1:length(D);
    for sloup = 1:1:length(kde);

        vstup = [D(rad) S kde(sloup)];
        pruch =  opruhuj_dna3(vstup,treatrot,maskrot);
        % pruch = [SID_object Area_object SID_stripes Area_stripes MS]
        krit = (pruch(3)/pruch(4))/(pruch(1)/pruch(2)); % relative intensity in stripes
        hods(rad,sloup) = krit; % ration of intensity inside and outside of stripes

    end
end

[rad sloup] = find(hods==max(hods(:))); % maximum relative intensity
rad = rad(1); sloup = sloup(1);
vstup = [D(rad) S kde(sloup)];
pruch =  opruhuj_dna3(vstup,treatrot,maskrot);
% pruch = [SID_object Area_object SID_stripes Area_stripes MS]
argout1 = pruch; 

