% Software for the evaluation of laser-induced stripes in cell nuclei.
% Positive Stripes
% 
% The method is described here:
% Martin Mistrik, Eva Vesela, et al:
% Cells and Stripes: A novel quantitative photo-manipulation technique
% Scientific Reports, to appear.

% called from: nactiprvni_dna1.m
% Main file: Segment_and_Track, see the documentation there

% This function segments a single window in the first image

function argout = segmentuj_dna1(argin1,argin2)

kus = argin1; % single window
pars = argin2; % parameters
   
kus = medfilt2(kus,[7 7]); % median filter
level = graythresh(kus); % otsu automatic threhold
bwkus = im2bw(kus,level); % produce logical mask
bwkus = bwmorph(bwkus,'open'); % postprocessing
bwkus = imfill(bwkus,'holes'); 

[L num] = bwlabel(bwkus,4);
stats = regionprops(L,'Centroid','Area','Solidity','PixelIdxList');
chars = zeros(num,5); % [dist_from_center, number, area, solidity, touching_boundary]
[ki kj] = size(kus);

for k=1:1:num
    chars(k,2) = k;
    pom = stats(k).Centroid;
    vzdal = (pom(1)-pars.del/2)^2 + (pom(2)-pars.del/2)^2 ; % dist_from_center
    chars(k,1) = sqrt(vzdal);
    chars(k,3) = stats(k).Area;
    chars(k,4) = stats(k).Solidity;
    pom = stats(k).PixelIdxList;
    [ii jj] = ind2sub(size(kus),pom);
    chars(k,5) = (min(ii)==1) || (min(jj)==1) || (max(ii) == ki) || (max(jj) == kj); % touching_boundary
end

kde = find(chars(:,5)==1); % these are touching the boundary
% throw them out and do it again
kusnew = kus;
for i=1:1:length(kde)
    ktere = stats(kde(i)).PixelIdxList;
	kusnew(ktere) = 0;
end

% do it again without the ones touching the boundary
% the code is just copied from above

kus = kusnew;

level = graythresh(kus);
bwkus = im2bw(kus,level); 
bwkus = bwmorph(bwkus,'open');
bwkus = imfill(bwkus,'holes');

[L num] = bwlabel(bwkus,4);
stats = regionprops(L,'Centroid','Area','Solidity','PixelIdxList');
chars = zeros(num,5); 

for k=1:1:num
    chars(k,2) = k;
    pom = stats(k).Centroid;
    vzdal = (pom(1)-pars.del/2)^2 + (pom(2)-pars.del/2)^2 ; 
    chars(k,1) = sqrt(vzdal);
    chars(k,3) = stats(k).Area;
    chars(k,4) = stats(k).Solidity;
    pom = stats(k).PixelIdxList;
    [ii jj] = ind2sub(size(kus),pom);
    chars(k,5) = (min(ii)==1) || (min(jj)==1) || (max(ii) == pars.del+1) || (max(jj) == pars.del+1); 
end

kde = find(chars(:,5)==1); % these are still touching the boundary
% throw them out and do it once more!
kusnew = kus;
for i=1:1:length(kde)
    ktere = stats(kde(i)).PixelIdxList;
	kusnew(ktere) = 0;
end

% do it for the third time without the ones touching the boundary
% the code is just copied from above

kus = kusnew;

level = graythresh(kus);
bwkus = im2bw(kus,level); 
bwkus = bwmorph(bwkus,'open');
bwkus = imfill(bwkus,'holes');

[L num] = bwlabel(bwkus,4);
stats = regionprops(L,'Centroid','Area','Solidity','PixelIdxList');
chars = zeros(num,5); 

for k=1:1:num
    chars(k,2) = k;
    pom = stats(k).Centroid;
    vzdal = (pom(1)-pars.del/2)^2 + (pom(2)-pars.del/2)^2 ;
    chars(k,1) = sqrt(vzdal);
    chars(k,3) = stats(k).Area;
    chars(k,4) = stats(k).Solidity;
    pom = stats(k).PixelIdxList;
    [ii jj] = ind2sub(size(kus),pom);
    chars(k,5) = (min(ii)==1) || (min(jj)==1) || (max(ii) == pars.del+1) || (max(jj) == pars.del+1); 
end

kde =  (chars(:,4)<pars.solid) | (chars(:,3)<pars.area) | (chars(:,5)==1); % the bad ones
chars(kde,:)=[]; % only the good ones are left
mask = false(size(kus));
for i=1:1:size(chars,1)
    mask = mask | L==chars(i,2);
end

argout = mask;