% Software for the evaluation of laser-induced stripes in cell nuclei.
% Negative Stripes
% 
% The method is described here:
% Martin Mistrik, Eva Vesela, et al:
% Cells and Stripes: A novel quantitative photo-manipulation technique
% Scientific Reports, to appear.

% Main file: Segment_and_Track, see the documentation there

% This functions allows the user to set the parameters.
%%

function argout = zadavani_dna

disp('All images must be in the "Data" folder');
disp('The images must be in the correct chrological order');
disp('The first image is taken for control at t=0.')

disp('**************************************************')
disp('SEGMENTATION PARAMETERS')
disp('**************************************************')

s = input('Minimum object area in pixels [default = 1000]: ', 's');
s = str2double(s);
if isnan(s) || (s<1)
    minarea = 1000;
else
    minarea = s;
end
disp(['Minimum area = ' num2str(minarea)])

disp('**************************************************')
s = input('Minimum solidity required from an object [default = 0.90]: ', 's');
s = str2double(s);
if isnan(s) || (s<=0) || (s>1)
    solidity = 0.90;
else
    solidity = s;
end
disp(['Minimum solidity = ' num2str(solidity)])

disp('**************************************************')
disp('STRIATION PARAMETERS')
disp('**************************************************')

s = input('Distance between the stripes in pixels [default = 32]: ', 's');
s = str2double(s);
if isnan(s) || s~=floor(s) || (s<=0)
    sopel = 32;
else
    sopel = s;
end
disp(['Distance between the stripes = ' num2str(sopel)])

s = input('Half-width of each stripe in pixels [default = 2]: ', 's');
s = str2double(s);
if isnan(s) || (s<=0) || s>=sopel/2
    polomer = 2;
else
    polomer = s;
end
disp(['Half-width of each stripe = ' num2str(polomer)])

pars.area = minarea;
pars.solid = solidity;
pars.sopel = sopel;
pars.polomer = polomer;

argout = pars;